@@ -8,10 +8,15 @@ import logging
 import os
 import re
 import textwrap
+import urllib.request
 from collections.abc import MutableMapping
 from typing import TYPE_CHECKING, Any
+from urllib.parse import urlparse
+
+from mkdocs.exceptions import BuildError
 
 from mkdocs_include_markdown_plugin import process
+from mkdocs_include_markdown_plugin.cache import Cache
 from mkdocs_include_markdown_plugin.config import (
     CONFIG_DEFAULTS,
     DEFAULT_CLOSING_TAG,
@@ -27,12 +32,7 @@ from mkdocs_include_markdown_plugin.regexes import (
 
 
 if TYPE_CHECKING:  # remove this for mypyc compiling
-    import sys
-
-    if sys.version_info >= (3, 8):
-        from typing import TypedDict
-    else:
-        from typing_extensions import TypedDict
+    from typing import TypedDict
 
     from mkdocs.structure.pages import Page
 
@@ -54,6 +54,15 @@ TRUE_FALSE_BOOL_STR = {
 }
 
 
+def is_url(string: str) -> bool:
+    """Determines if a string is a URL."""
+    try:
+        result = urlparse(string)
+        return all([result.scheme, result.netloc])
+    except ValueError:
+        return False
+
+
 def bool_arg(arg: str) -> re.Pattern[str]:
     """Return a compiled regexp to match a boolean argument."""
     return re.compile(rf'{arg}=(\w+)')
@@ -124,6 +133,20 @@ def read_file(file_path: str, encoding: str) -> str:
         return f.read()
 
 
+def read_http(target_url: str, http_cache: Cache | None) -> Any:  # noqa: U100
+    """Read an http location and return its content."""
+    if http_cache is not None:
+        cached_content = http_cache.get_(target_url)
+        if cached_content is not None:
+            return cached_content
+    req = urllib.request.Request(target_url)
+    with urllib.request.urlopen(req) as response:
+        content = response.read().decode('UTF-8')
+    if http_cache is not None:
+        http_cache.set_(target_url, content)
+    return content
+
+
 def get_file_content(
     markdown: str,
     page_src_path: str,
@@ -137,6 +160,7 @@ def get_file_content(
     default_comments: bool = DEFAULT_COMMENTS,
     cumulative_heading_offset: int = 0,
     files_watcher: FilesWatcher | None = None,
+    http_cache: Cache | None = None,
 ) -> str:
     """Return the content of the file to include."""
     def found_include_tag(match: re.Match[str]) -> str:
@@ -150,17 +174,18 @@ def get_file_content(
                 markdown,
                 directive_match_start,
             )
-            logger.error(
+            raise BuildError(
                 "Found no path passed including with 'include'"
                 f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                 f':{lineno}',
             )
-            return ''
 
         arguments_string = match.group('arguments')
 
         if os.path.isabs(filename):
             file_path_glob = filename
+        elif is_url(filename):
+            file_path_glob = filename
         else:
             file_path_glob = os.path.join(
                 os.path.abspath(os.path.dirname(page_src_path)),
@@ -177,11 +202,10 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'exclude' argument in 'include' directive"
                     f' at {os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
-                ignore_paths = []
             else:
 
                 if os.path.isabs(exclude_string):
@@ -199,20 +223,22 @@ def get_file_content(
             glob.iglob(file_path_glob, recursive=True),
             ignore_paths=ignore_paths,
         )
+        if is_url(filename):
+            file_paths_to_include = [file_path_glob]
 
         if not file_paths_to_include:
             lineno = lineno_from_content_start(
                 markdown,
                 directive_match_start,
             )
-            logger.error(
+            raise BuildError(
                 f"No files found including '{raw_filename}'"
                 f' at {os.path.relpath(page_src_path, docs_dir)}'
                 f':{lineno}',
             )
-            return ''
         elif files_watcher is not None:
-            files_watcher.included_files.extend(file_paths_to_include)
+            if not is_url(file_path_glob):
+                files_watcher.included_files.extend(file_paths_to_include)
 
         bool_options: dict[str, DirectiveBoolArgument] = {
             'preserve-includer-indent': {
@@ -244,12 +270,11 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     f"Invalid value for '{arg_name}' argument of 'include'"
                     f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                     f':{lineno}. Possible values are true or false.',
                 )
-                return ''
 
         start_match = ARGUMENT_REGEXES['start'].search(arguments_string)
         if start_match:
@@ -259,7 +284,7 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'start' argument in 'include' directive at "
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
@@ -274,7 +299,7 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'end' argument in 'include' directive at "
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
@@ -289,19 +314,21 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'encoding' argument in 'include'"
                     ' directive at '
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
                 )
-                encoding = 'utf-8'
         else:
             encoding = default_encoding
 
         text_to_include = ''
         expected_but_any_found = [start is not None, end is not None]
         for file_path in file_paths_to_include:
-            new_text_to_include = read_file(file_path, encoding)
+            if is_url(filename):
+                new_text_to_include = read_http(file_path, http_cache)
+            else:
+                new_text_to_include = read_file(file_path, encoding)
 
             if start is not None or end is not None:
                 new_text_to_include, *expected_not_found = (
@@ -327,6 +354,7 @@ def get_file_content(
                 default_dedent,
                 default_trailing_newlines,
                 files_watcher=files_watcher,
+                http_cache=http_cache,
             )
 
             # trailing newlines right stripping
@@ -386,17 +414,18 @@ def get_file_content(
                 markdown,
                 directive_match_start,
             )
-            logger.error(
+            raise BuildError(
                 "Found no path passed including with 'include-markdown'"
                 f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                 f':{lineno}',
             )
-            return ''
 
         arguments_string = match.group('arguments')
 
         if os.path.isabs(filename):
             file_path_glob = filename
+        elif is_url(filename):
+            file_path_glob = filename
         else:
             file_path_glob = os.path.join(
                 os.path.abspath(os.path.dirname(page_src_path)),
@@ -413,12 +442,11 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'exclude' argument in 'include-markdown'"
                     f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                     f':{lineno}',
                 )
-                ignore_paths = []
             else:
                 if os.path.isabs(exclude_string):
                     exclude_globstr = exclude_string
@@ -436,19 +464,22 @@ def get_file_content(
             ignore_paths=ignore_paths,
         )
 
+        if is_url(filename):
+            file_paths_to_include = [file_path_glob]
+
         if not file_paths_to_include:
             lineno = lineno_from_content_start(
                 markdown,
                 directive_match_start,
             )
-            logger.error(
+            raise BuildError(
                 f"No files found including '{raw_filename}' at"
                 f' {os.path.relpath(page_src_path, docs_dir)}'
                 f':{lineno}',
             )
-            return ''
         elif files_watcher is not None:
-            files_watcher.included_files.extend(file_paths_to_include)
+            if not is_url(file_path_glob):
+                files_watcher.included_files.extend(file_paths_to_include)
 
         bool_options: dict[str, DirectiveBoolArgument] = {
             'rewrite-relative-urls': {
@@ -488,13 +519,12 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     f"Invalid value for '{arg_name}' argument of"
                     " 'include-markdown' directive at"
                     f' {os.path.relpath(page_src_path, docs_dir)}'
                     f':{lineno}. Possible values are true or false.',
                 )
-                return ''
 
         # start and end arguments
         start_match = ARGUMENT_REGEXES['start'].search(arguments_string)
@@ -505,7 +535,7 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'start' argument in 'include-markdown'"
                     f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                     f':{lineno}',
@@ -521,7 +551,7 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'end' argument in 'include-markdown'"
                     f' directive at {os.path.relpath(page_src_path, docs_dir)}'
                     f':{lineno}',
@@ -537,7 +567,7 @@ def get_file_content(
                     markdown,
                     directive_match_start,
                 )
-                logger.error(
+                raise BuildError(
                     "Invalid empty 'encoding' argument in 'include-markdown'"
                     ' directive at '
                     f'{os.path.relpath(page_src_path, docs_dir)}:{lineno}',
@@ -570,7 +600,10 @@ def get_file_content(
 
         text_to_include = ''
         for file_path in file_paths_to_include:
-            new_text_to_include = read_file(file_path, encoding)
+            if is_url(filename):
+                new_text_to_include = read_http(file_path, http_cache)
+            else:
+                new_text_to_include = read_file(file_path, encoding)
 
             if start is not None or end is not None:
                 new_text_to_include, *expected_not_found = (
@@ -597,6 +630,7 @@ def get_file_content(
                 default_trailing_newlines,
                 default_comments=default_comments,
                 files_watcher=files_watcher,
+                http_cache=http_cache,
             )
 
             # trailing newlines right stripping
@@ -687,6 +721,7 @@ def on_page_markdown(
     docs_dir: str,
     config: MutableMapping[str, Any] | None = None,
     files_watcher: FilesWatcher | None = None,
+    http_cache: Cache | None = None,
 ) -> str:
     """Process markdown content of a page."""
     if config is None:
@@ -720,4 +755,5 @@ def on_page_markdown(
         config.get('trailing_newlines', CONFIG_DEFAULTS['trailing_newlines']),
         default_comments=config.get('comments', CONFIG_DEFAULTS['comments']),
         files_watcher=files_watcher,
+        http_cache=config.get('_cache', http_cache),
     )
