@@ -3,25 +3,19 @@
 from __future__ import annotations
 
 from collections.abc import Callable
-from typing import TYPE_CHECKING, Any, TypeVar, cast
+from typing import TYPE_CHECKING, Any, cast
 
+from mkdocs.exceptions import BuildError
 from mkdocs.livereload import LiveReloadServer
-from mkdocs.plugins import BasePlugin
+from mkdocs.plugins import BasePlugin, event_priority
 
 
-try:
-    from mkdocs.plugins import event_priority
-except ImportError:
-    T = TypeVar('T')
-
-    def event_priority(priority: float) -> Callable[[T], T]:  # noqa: D103
-        return lambda f: f
-
 if TYPE_CHECKING:
     from mkdocs.config.defaults import MkDocsConfig
     from mkdocs.structure.files import Files
     from mkdocs.structure.pages import Page
 
+from mkdocs_include_markdown_plugin.cache import initialize_cache
 from mkdocs_include_markdown_plugin.config import (
     CONFIG_SCHEME,
     create_include_tag,
@@ -51,6 +45,19 @@ class IncludeMarkdownPlugin(BasePlugin):
             tag='include-markdown',
         )
 
+        if self.config['cache'] > 0:
+            cache = initialize_cache(self.config['cache'])
+            if cache is None:
+                raise BuildError(
+                    'The "platformdirs" package is required to use the'
+                    ' "cache" option. Install'
+                    ' mkdocs-include-markdown-plugin with the "cache"'
+                    ' extra to install it.',
+                )
+            else:
+                cache.clean()
+                self.config['_cache'] = cache
+
         return config
 
     def _watch_included_files(self) -> None:
